/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.core.query.lucene;

import java.io.IOException;

/**
 * <code>RedoLogFactory</code> a factory for {@link RedoLog}s.
 */
public interface RedoLogFactory {

    /**
     * Creates a redo log starting at the current state of the
     * <code>index</code>.
     *
     * @param index the index.
     * @return the redo log.
     * @throws IOException if an error occurs while reading from the index or
     *                     the log cannot be created for some other reason.
     */
    public RedoLog createRedoLog(MultiIndex index) throws IOException;
}
