/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.core.query.lucene;

import junit.framework.TestCase;

public class SearchIndexTest extends TestCase {

    /**
     * @see <a href="https://issues.apache.org/jira/browse/JCR-3236">JCR-3236</a>
     */
    public void testSetAnalyzer() {
        String[] analyzers = {
                "org.apache.lucene.analysis.SimpleAnalyzer",
                "org.apache.lucene.analysis.StopAnalyzer",
                "org.apache.lucene.analysis.standard.StandardAnalyzer" };
        SearchIndex index = new SearchIndex();
        for (String analyzer : analyzers) {
            index.setAnalyzer(analyzer);
            assertEquals(analyzer, index.getAnalyzer());
        }
    }

}
