/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.spi.commons.query.qom;

import javax.jcr.query.qom.BindVariableValue;

import org.apache.jackrabbit.spi.commons.conversion.NamePathResolver;
import org.apache.jackrabbit.spi.Name;

/**
 * <code>BindVariableValueImpl</code>...
 */
public class BindVariableValueImpl
        extends StaticOperandImpl
        implements BindVariableValue {

    /**
     * The name of the bind variable.
     */
    private final Name variableName;

    BindVariableValueImpl(NamePathResolver resolver, Name variableName) {
        super(resolver);
        this.variableName = variableName;
    }

    /**
     * Gets the name of the bind variable.
     *
     * @return the bind variable name; non-null
     */
    public Name getBindVariableQName() {
        return variableName;
    }

    //-------------------------< BindVariableValue >----------------------------

    /**
     * Gets the name of the bind variable.
     *
     * @return the bind variable name; non-null
     */
    public String getBindVariableName() {
        return getJCRName(variableName);
    }

    //------------------------< AbstractQOMNode >-------------------------------

    /**
     * Accepts a <code>visitor</code> and calls the appropriate visit method
     * depending on the type of this QOM node.
     *
     * @param visitor the visitor.
     */
    public Object accept(QOMTreeVisitor visitor, Object data) throws Exception {
        return visitor.visit(this, data);
    }

    //------------------------< Object >----------------------------------------

    public String toString() {
        return "$" + getBindVariableName();
    }

}
