/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.server.remoting.davex;

import javax.jcr.Item;
import javax.jcr.Node;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.security.AccessControlManager;
import javax.jcr.security.AccessControlPolicy;

import org.apache.jackrabbit.util.Text;

public class AclRemoveHandler implements ProtectedItemRemoveHandler {

    private static final String NT_REP_ACL = "rep:ACL";

    @Override
    public boolean remove(Session session, String itemPath) throws RepositoryException {
        if (canHandle(session, itemPath)) {
            String controlledPath = Text.getRelativeParent(itemPath, 1);
            AccessControlManager acMgr = session.getAccessControlManager();
            AccessControlPolicy[] policies = acMgr.getPolicies(controlledPath);
            for (AccessControlPolicy policy : policies) {
                acMgr.removePolicy(controlledPath, policy);
            }
            return true;
        }
        return false;
    }

    // -----------------------------------------------------------< private >---
    private boolean canHandle(Session session, String itemPath) throws RepositoryException {
        Item aclItem = session.getItem(itemPath);
        if (aclItem.isNode() && itemPath.startsWith("/")) {
            if (isJackrabbitAclNodeType((Node) aclItem)) {
                return true;
            }
        }
        return false;
    }

    private boolean isJackrabbitAclNodeType(Node aclNode) throws RepositoryException {
        String ntName = aclNode.getPrimaryNodeType().getName();
        return ntName.equals(NT_REP_ACL);
    }
}
